#! /bin/sh 
#
# $Id: autogen.sh,v 1.13 2003/04/17 22:50:54 jsquyres Exp $
#
# This script is run on developer copies of LAM/MPI -- *not*
# distribution tarballs.
#
# Some helper functions
#

#
# Subroutine to check for the existence of various standard GNU tools
#
test_for_existence() {
    tfe_prog="$1"
    tfe_foo="`$tfe_prog --version`"
    if test "$?" != 0; then
	cat <<EOF

You must have GNU autoconf, automake, and libtool installed to build
the developer's version of LAM/MPI.  You can obtain these packages
from ftp://ftp.gnu.org/gnu/.

EOF
	# Stupid emacs: '
	exit 1
    fi
    unset tfe_prog tfe_foo
}


#
# Subroutine to execite the standard GNU tools, and if they fail,
# print out a warning.
#
run_and_check() {
    rac_progs="$*"
    echo "$rac_progs"
    eval $rac_progs
    if test "$?" != 0; then
	cat <<EOF

It seems that the execution of "$progs" has failed.
I am gonna abort.  :-(

This may be caused by an older version of one of the required
packages.  Please make sure you are using at least the following
versions:

GNU Autoconf 2.52
GNU Automake 1.5
GNU Libtool  1.4.2

EOF
	exit 1
    fi
    unset rac_progs
}

#
# Subroutine to look for standard files in a number of common places
# (e.g., ./config.guess, config/config.guess, dist/config.guess), and
# delete it.  If it's not found there, look for AC_CONFIG_AUX_DIR in
# the configure.in script and try there.  If it's not there, oh well.
#
find_and_delete() {
    fad_file="$1"

    # Look for the file in "standard" places

    if test -f $fad_file; then
	rm -f $fad_file
    elif test -d config/$fad_file; then
	rm -f config/$fad_file
    elif test -d dist/$fad_file; then
	rm -f dist/$fad_file
    else

	# Didn't find it -- look for an AC_CONFIG_AUX_DIR line in
	# configure.[in|ac]

	if test -f configure.in; then
	    fad_cfile=configure.in
	elif test -f configure.ac; then
	    fad_cfile=configure.ac
	else
	    echo "--> Errr... there's no configure.in or configure.ac file!"
	fi
	if test -n "$fad_cfile"; then
	    auxdir="`grep AC_CONFIG_AUX_DIR $fad_cfile | cut -d\( -f 2 | cut -d\) -f 1`"
	fi	
	if test -f "$auxdir/$fad_file"; then
	    rm -f "$auxdir/$fad_file"
	fi
	unset fad_cfile
    fi
    unset fad_file
}


#
# Subroutine to actually do the GNU tool setup in the proper order, etc.
#
run_gnu_tools() {
    rgt_dir="$1"
    rgt_cur_dir="`pwd`"
    if test -d "$rgt_dir"; then
	cd "$rgt_dir"

	# See if the package doesn't want us to set it up

	if test -f .lam_no_gnu; then
	    cat <<EOF

*** Found .lam_no_gnu file -- skipping GNU setup in:
***   `pwd`

EOF
	elif test -f .lam_ignore; then
	    cat <<EOF

*** Found .lam_ignore file -- skipping entire tree:
***   `pwd`

EOF
	elif test "$rgt_dir" != "." -a -x autogen.sh; then
	    cat <<EOF

*** Found custom autogen.sh file in:
***   `pwd`

EOF
	    ./autogen.sh
        else
	    cat <<EOF

*** Running GNU tools in directory: 
***   `pwd`

EOF
	    # Sanity check to ensure that there's a configure.in or
	    # configure.ac file here

	    if test -f configure.in; then
		happy=1
		file=configure.in
	    elif test -f configure.ac; then
		happy=1
		file=configure.ac
	    else
		echo "---> Err... there's no configure.in or configure.ac file in this directory"
		echo "---> I'm confused, so I'm going to abort"
		exit 1
	    fi
	    unset happy

	    # Find and delete the GNU helper script files

	    find_and_delete config.guess
	    find_and_delete config.sub
	    find_and_delete depcomp
	    find_and_delete install-sh
	    find_and_delete ltconfig
	    find_and_delete ltmain.sh
	    find_and_delete missing
	    find_and_delete mkinstalldirs
	    find_and_delete libtool

            # Run the GNU tools

	    run_and_check aclocal
	    if test "`grep AC_CONFIG_HEADER $file`" != "" -o \
		"`grep AM_CONFIG_HEADER $file`" != ""; then
		run_and_check autoheader
	    fi
	    run_and_check autoconf
	    run_and_check libtoolize --automake --copy
	    run_and_check automake --foreign -a --copy --include-deps
	fi
	
	# Go back to the original directory

	cd "$rgt_cur_dir"
    fi
    unset rgt_dir rgt_cur_dir
}


#
# Subroutine to run in just one directory
#
run_local() {
    run_gnu_tools .
}


#
# Subroutine to run across the entire LAM tree
#
run_global() {
    # Run the config in the top-level directory

    run_gnu_tools .

    # Now run the config in every directory in share/ssi/*/* that has
    # a configure.in or configure.ac script

    for kind in share/ssi/*; do
	if test -d "$kind"; then
	    for module in "$kind"/*; do
		if test -d "$module"; then
		    if test -f "$module/configure.in" -o -f "$module/configure.ac"; then
			run_gnu_tools "$module"
		    fi
		fi
	    done
	fi
    done
    unset kind module

    # Run autogen.sh in share/totalview

    run_gnu_tools share/totalview
}


##########################################################################
# Main
##########################################################################

# Check the command line to see if we should run the whole shebang, or
# just in this current directory.

want_local=0
for arg in $*; do
    case $arg in
    -l) want_local=1 ;;
    *) ;;
    esac
done

# Are we in the right directory?  We must be in the top-level LAM
# directory.

if test "$want_local" = "0"; then
    if test -f VERSION -a -f configure.in -a -f share/mpi/init.c ; then
	bad=0
    else
	cat <<EOF

You must run this script from the top-level LAM directory.

EOF
	exit 1
    fi
fi

test_for_existence autoconf
test_for_existence automake
test_for_existence libtool

# Now do the run

if test "$want_local" = "1"; then
    run_local
else
    run_global
fi

# All done

exit 0
